using System;
using System.IO;
using System.Text;
using System.Linq;
using System.ComponentModel;
using System.Collections.Generic;
using System.Windows.Forms;

using SwissAcademic.Citavi;
using SwissAcademic.Citavi.Metadata;
using SwissAcademic.Citavi.Shell;
using SwissAcademic.Collections;

// Implementation of macro editor is preliminary and experimental.
// The Citavi object model is subject to change in future version.

public static class CitaviMacro
{
	public static void Main()
	{
		StringBuilder builder = new StringBuilder();
		
		// Keywords
		ProjectKeywordCollection keywords = Program.ActiveProjectShell.Project.Keywords; 
		List<string> fullnames = new List<string>();
		
		foreach(Keyword k in keywords)
		{
			var value = KeywordPropertyDescriptor.Name.Clean(k.FullName); 

			if(fullnames.Contains(value))
			{
				builder.AppendLine("Duplicate Keyword: " + value);
			}
			else
			{
				fullnames.Add(value);
			}
		}
		
		// Persons
		ProjectPersonCollection persons = Program.ActiveProjectShell.Project.Persons;
		fullnames.Clear();
		
		foreach(Person p in persons)
		{
			var value = PersonPropertyDescriptor.FirstName.Clean(p.FullName);
			
			if(fullnames.Contains(value))
			{
				builder.AppendLine("Duplicate Person: " + value);
			}
			else
			{
				fullnames.Add(value);
			}
		}
		
		// Periodical
		ProjectPeriodicalCollection periodicals = Program.ActiveProjectShell.Project.Periodicals;
		fullnames.Clear();
		
		foreach(Periodical p in periodicals)
		{
			var value = PeriodicalPropertyDescriptor.Name.Clean(p.FullName);
			
			if(fullnames.Contains(value))
			{
				builder.AppendLine("Duplicate Periodical: " + value);
			}
			else
			{
				fullnames.Add(value);
			}
		}
		
		// Library
		ProjectLibraryCollection libraries = Program.ActiveProjectShell.Project.Libraries;
		fullnames.Clear();
		
		foreach(Library l in libraries)
		{
			var value = LibraryPropertyDescriptor.Name.Clean(l.FullName);
			
			if(fullnames.Contains(value))
			{
				builder.AppendLine("Duplicate Library: " + value);
			}
			else
			{
				fullnames.Add(value);
			}
		}
		
		// Publisher
		ProjectPublisherCollection publishers = Program.ActiveProjectShell.Project.Publishers;
		fullnames.Clear();
		
		foreach(Publisher p in publishers)
		{
			var value = PublisherPropertyDescriptor.Name.Clean(p.FullName);
			
			if(fullnames.Contains(value))
			{
				builder.AppendLine("Duplicate Publisher: " + value);
			}
			else
			{
				fullnames.Add(value);
			}
		}
		
		// SeriesTitle
		ProjectSeriesTitleCollection seriesTitles = Program.ActiveProjectShell.Project.SeriesTitles;
		fullnames.Clear();
		
		foreach(SeriesTitle s in seriesTitles)
		{
			var value = SeriesTitlePropertyDescriptor.Name.Clean(s.FullName);
			
			if(fullnames.Contains(value))
			{
				builder.AppendLine("Duplicate SeriesTitle: " + value);
			}
			else
			{
				fullnames.Add(value);
			}
		}
		
		// Groups
		ProjectGroupCollection groups = Program.ActiveProjectShell.Project.Groups;
		fullnames.Clear();
		
		foreach(Group g in groups)
		{
			var value = GroupPropertyDescriptor.Name.Clean(g.FullName);
			
			if(fullnames.Contains(value))
			{
				builder.AppendLine("Duplicate Group: " + value);
			}
			else
			{
				fullnames.Add(value);
			}
		}
		
		if(string.IsNullOrEmpty(builder.ToString()))
		{
			MessageBox.Show("No duplicates found");
		}
		else
		{
			var orignalValue = builder.ToString();
			
			builder.AppendLine();
			builder.AppendLine("Save results?");
			
			if(MessageBox.Show(builder.ToString(), "Found duplicates", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
			{
				using(FolderBrowserDialog dialog = new FolderBrowserDialog())
				{	
					if(dialog.ShowDialog() == DialogResult.OK)
					{
						try
						{	
							using(FileStream stream = new FileStream(Path.Combine(dialog.SelectedPath, string.Format("Find_Duplicates_Result_{0}.txt", DateTime.Now.ToString("dd-MM-yyyy HH-mm-ss"))), FileMode.OpenOrCreate, FileAccess.ReadWrite))
							{
								using(StreamWriter wrtr = new StreamWriter(stream))
								{
									wrtr.Write(orignalValue);
								}
							}
						}
						catch(Exception x)
						{
							MessageBox.Show(x.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
						}
					}
				}
			}
		}
	}
}